#include "hal_uart1.h"

#define HAL_UART1_TX_PORT   GPIOA
#define HAL_UART1_TX_PIN    GPIO_Pin_9
#define HAL_UART1_TX_SOURCE GPIO_PinSource9
#define HAL_UART1_RX_PORT   GPIOA
#define HAL_UART1_RX_PIN    GPIO_Pin_10
#define HAL_UART1_RX_SOURCE GPIO_PinSource10

static uint8_t uart1_tx_buf[UART1_TX_LEN]  = {0};
static uint8_t uart1_rx_buf[UART1_REC_LEN] = {0};

__IO uint8_t rx_index = 0x00;

__IO uint8_t tx_index = 0x00;

__IO uint8_t TimeOut = 0x00;

//  重定义fputc函数
#ifdef __GNUC__

int _write(int fd, char *buffer, int size)
{
    /* 写一个字节到USART1 */
    for (int i = 0; i < size; i++)
    {
        USART_SendData(USART1, buffer[i]);
        /* 等待发送结束 */
        while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
            ;
    }

    return size;
}
int _read(int fd, char *buffer, int size)
{

    for (int i = 0; i < size; i++)
    {
        /* 等待串口1输入数据 */
        while (USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == RESET)
            ;
        buffer[i] = USART_ReceiveData(USART1);
    }
    return size;
}
#endif

void hal_uart1_nvic_config(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;
    // Usart1 NVIC 配置
    NVIC_InitStructure.NVIC_IRQChannel                   = USART1_IRQn; // 串口1中断通道
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;           // 抢占优先级
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 0;           // 子优先级
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE;      // IRQ通道使能
    NVIC_Init(&NVIC_InitStructure);                                     // 根据指定的参数初始化VIC寄存器
}

void hal_uart1_gpio_config(void)
{
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
    GPIO_InitTypeDef GPIO_StructInit;
    GPIO_StructInit.GPIO_Mode  = GPIO_Mode_AF;
    GPIO_StructInit.GPIO_OType = GPIO_OType_PP;
    GPIO_StructInit.GPIO_Speed = GPIO_Speed_100MHz;
    GPIO_StructInit.GPIO_PuPd  = GPIO_PuPd_UP;
    GPIO_StructInit.GPIO_Pin   = HAL_UART1_TX_PIN;
    GPIO_PinAFConfig(HAL_UART1_TX_PORT, HAL_UART1_TX_SOURCE, GPIO_AF_USART1);
    GPIO_Init(HAL_UART1_TX_PORT, &GPIO_StructInit);

    GPIO_StructInit.GPIO_Pin = HAL_UART1_RX_PIN;
    GPIO_PinAFConfig(HAL_UART1_RX_PORT, HAL_UART1_RX_SOURCE, GPIO_AF_USART1);
    GPIO_Init(HAL_UART1_RX_PORT, &GPIO_StructInit);
}
void hal_uart1_config(void)
{

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

    USART_InitTypeDef USART_InitStructure;
    I2C_DeInit(I2C1);

    USART_InitStructure.USART_BaudRate            = 9600;
    USART_InitStructure.USART_WordLength          = USART_WordLength_8b;
    USART_InitStructure.USART_WordLength          = USART_StopBits_1;
    USART_InitStructure.USART_Parity              = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode                = USART_Mode_Rx | USART_Mode_Tx;

    USART_Init(USART1, &USART_InitStructure);

    /* 使能 USART */
    USART_Cmd(USART1, ENABLE);
}

void hal_uart1_init(void)
{
    hal_uart1_gpio_config();
    hal_uart1_config();
    hal_uart1_nvic_config();

    USART_ClearFlag(USART1, USART_FLAG_TC);
}

void USART1_IRQHandler(void)
{
    /* 处理接收中断 */
    if (USART_GetITStatus(USART1, USART_IT_RXNE) == SET)
    {
        if (rx_index < UART1_REC_LEN)
        {
            /* 接收数据 */
            uart1_rx_buf[rx_index++] = USART_ReceiveData(USART1);
        }
        else
        {
            /* 禁止接收中断 */
            USART_ITConfig(USART1, USART_IT_RXNE, DISABLE);
        }
    }
    /* 处理发送缓冲区空中断 */
    if (USART_GetITStatus(USART1, USART_IT_TXE) == SET)

    {
        if (tx_index < UART1_TX_LEN)
        {
            /* 发送数据 */
            USART_SendData(USART1, uart1_tx_buf[tx_index++]);
        }
        else
        {
            /* 禁止发送缓冲区空中断 */
            USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
        }
    }
}